function md_denoise_conn_comp(x,num_knn,lambda,timestep,GraphType,num_iter)
% Denoises manifold data via a diffusion process induced by the graph
% Laplacian and computes simultaneously the number of connected components
% of the k-nearest neighbor graph. This information could be used in a
% stopping criterion.
%   x        :  points
%   lambda   :  reweighting of the weights) with the degree function
%               \tilde{k}_ij = k_ij / pow(d_i d_j, lambda)
%   num_knn  :  number of k-nearest neighbors
%   num_iter :  number of iterations
%   timestep :  explicit=0, implicit=1
%   GraphType:  Type of graph (0: mutal, 1:symmetric, 2: other symmetric, see
%               getSparseWeightMatrix... .cpp for a better description

% number of data points and dimension
[dim num] = size(x);

for iter=1:num_iter
    % display number of iteration
    disp(['Iteration:',num2str(iter)]);

    % do the timestep and compute 
    old_x = x;
    [x,KNN,KNNDist,K] = md_time_step(x,num_knn,lambda,timestep,GraphType);
    diff_x = x - old_x;

    %% statistics that are not essential to the denoising algorithm
    % compute the kNN distance histogram
    [counts,binloc]=hist(reshape(KNNDist,num*num_knn,1),100);

    % examine connected components
    compvec = getComps(K);
    numComps = max(compvec);
    disp(['Number of connected components: ', num2str(max(compvec))]);
    numPointsInComps = zeros(numComps,1);
    for i=1:numComps
        numPointsInComps(i)=nnz(compvec==i);
    end
    numPointsInComps=sort(numPointsInComps,'descend');
    disp(['Number of points in the ',num2str(min(numComps,10)),' largest components']);
    for i=1:min(numComps,10)
        disp(['Cluster ',num2str(i),' with ',num2str(numPointsInComps(i)),' points']);
    end

    % clear the matrix of KNN distances
    clear KNNDist;

    % plot all the information
    md_plot_points_dists(x,old_x,diff_x,binloc,counts,iter,lambda,num_knn);

end