#include <math.h>
#include "mex.h"

void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[]) {
  // Test number of parameters.
  if (nrhs != 5 || nlhs != 1) {
    mexWarnMsgTxt("Usage: K = getSparseWeightMatrix(dist, knDist, radius, knNum,GraphType)");
    return;
  }
  // Parse parameters
  int num = (int)mxGetM(prhs[0]);
  double* dists = mxGetPr(prhs[0]);
  double* knDist = mxGetPr(prhs[1]);
  double radius = mxGetScalar(prhs[2]);
  int knNum = (int) (mxGetScalar(prhs[3])+0.1);
  int mode  = (int) (mxGetScalar(prhs[4])); // 0: mututal k-nearest neighbor, 1: maximal symmetric k-nearest neighbor, 2:average

  // small positive constant
  double one_plus_eps = 1 + 0.0000000001;

  // Allocate memory for output (sparse real matrix)
  int m = num;
  int n = num;
  int nzmax;
  if(mode!=0) nzmax = 2*num*knNum; // since we have a symmetric neighborhood graph there can be more than
                           // num*knNum entries in the matrix, 2*num*knNum is an upper bound
                           // typically in high dimensions one has 1.5*num*knNum, however we use
                           // here the upper bound, since otherwise one has to allocate new memory
                           // which is time expensive.
  else // in the case of a mutual knn-graph we have less than k edges from each vertex
    nzmax = num*knNum;
  plhs[0] = mxCreateSparse(num, num, nzmax, mxREAL);
  double* sr = mxGetPr(plhs[0]);
  mwIndex* irs = mxGetIr(plhs[0]);
  mwIndex* jcs = mxGetJc(plhs[0]);


  double dist,arg1,arg2,Arg;
  int k = 0,i,j;
  for(j=0; j<n; j++) {    // loop over columns
    jcs[j] = k;
    for(i=0; i<m; i++) {     // loop over rows
      if(i!=j) {
	// get the distance
	dist = dists[i+j*num];

	// distance relevant?
	arg1 = dist/(knDist[i]*radius);
	arg2 = dist/(knDist[j]*radius);
        switch(mode)
        {
          case 0: Arg = arg1;
	          if(arg2>arg1)
	            Arg = arg2;
                  break;
          case 1: Arg = arg1;
                  if(arg2<arg1)
                    Arg = arg2;
                  break;
          case 2: Arg=0.5*(arg1+arg2); break;
        }

	if(Arg<one_plus_eps) {
	  if(k>=nzmax) {
	    nzmax += num;
	    mxSetNzmax(plhs[0],nzmax);
	    mxSetPr(plhs[0],(double*)mxRealloc(sr,nzmax*sizeof(double)));
	    mxSetIr(plhs[0],(mwIndex*)mxRealloc(irs,nzmax*sizeof(int)));
	    sr = mxGetPr(plhs[0]);
	    irs = mxGetIr(plhs[0]);
	  }
	  sr[k] = exp(-Arg*Arg);
	  irs[k] = i;
	  k++;	
	}
      }
    }
  }
  jcs[n] = k;
}
